c     Copyright (C)
c     2000 -2013 Universite Paul Sabatier,
c     2000 -2013 Centre National de la Recherche Scientifique
c     2000 -2013 Ecole des Mines d'Albi
c     2014-2018 Msostar
c
c     This program is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 2, or (at your option)
c     any later version.

c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with this program; if not, write to the Free Software
c     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
c     USA.
c
      subroutine init_kcoeff(Nb,Nq)
      implicit none

c     Cette routine va chercher les valeurs du coefficient kbar dans les diffrentes
c     bandes spectrales et  toutes les altitudes

      include 'files_in.inc'
      include 'max.inc'
      include 'data.inc'
      include 'options.inc'
      include 'formats.inc'
      integer i,Nb,Nq,band,quad,raf1
      integer l_min_t,l_max_t
      double precision lambda_min,lambda_max

      l_min_t=0
      l_max_t=0
      
c     Rcupration des donnes spectrales et remise en ordre...
      open(10,file=file_ray)

      read(10,*) raf1,Nb,Nq
c     Tests sur Nb et Nq
      if (Nb.gt.Nbmx) then
         call screen(1404,Nb,Nbmx,0.0D+0,0.0D+0)
         call screen(15,0,0,0.0D+0,0.0D+0)
         call screen(44,0,0,0.0D+0,0.0D+0)
         stop
      endif
      if (Nq.gt.Nqmx) then
         call screen(1405,Nq,Nqmx,0.0D+0,0.0D+0)
         call screen(15,0,0,0.0D+0,0.0D+0)
         call screen(44,0,0,0.0D+0,0.0D+0)
         stop
      endif
      
      do i=1,9 ! On passe directement aux donnes spectrales
         read(10,*)
      enddo

      if (ces.eq.1) then
c     Si on veut faire un calcul sur l'ensemble du spectre pour lequel on a les donnes
         nb_min=1
         nb_max=Nb
         l_min_t=1
         l_max_t=1
      endif
c     Debug
c      write(*,*) 'm=',m,' Nb=',Nb,' Nq=',Nq
c     Debug

      do band=1,Nb ! dbut boucle sur bandes
         read(10,50) lambda_lo(band),lambda_hi(band),
     &        em_sol(band),em_esp(band),(w_moy(band,quad),quad=1,Nq),
     &        udif_sol(band),udif_esp(band)

c     Je teste pour vrifier la cohrence des intervalles spectraux
         if (band.gt.1) then
            if (lambda_lo(band).ne.lambda_hi(band-1)) then
               call screen(1111,band-1,band,0.0D+0,0.0D+0)
               call screen(15,0,0,0.0D+0,0.0D+0)
               stop
            endif
         endif
         
         if (ces.eq.2) then
c     Si on veut faire un calcul sur une partie du spectre, il faut tester
c     pour reprer l'intervalle d'intgration demand
            if ((lambda_lo(band).le.l_min).and.
     &           (lambda_hi(band).ge.l_min).and.
     &           (l_min_t.eq.0)) then
               nb_min=band
               l_min_t=1
            endif
            if ((((lambda_lo(band).le.l_max).and.
     &           (lambda_hi(band).ge.l_max)).or.(band.eq.Nb))
     &           .and.(l_max_t.eq.0)) then
               nb_max=band
               l_max_t=1
            endif
         endif

         
         do quad=1,Nq ! Lecture des paisseurs optiques de chaque couche
c     Les lignes commentes c'est pour conomiser de la RAM
c     mais je les laisse pour indiquer ce qui se trouve dans le fichier
c            read(10,50) (k_gaz(band,i,quad),i=1,m)
            read(10,50)
            read(10,50) (ka_gaz(band,i,quad),i=1,m)
            read(10,50) (kd_gaz(band,i,quad),i=1,m)
c            read(10,50) (w_gaz(band,i,quad),i=1,m)
            read(10,50)
            read(10,50) (g_gaz(band,i,quad),i=1,m)
c            read(10,50) (k_aero(band,i,quad),i=1,m)
            read(10,50)
            read(10,50) (ka_aero(band,i,quad),i=1,m)
            read(10,50) (kd_aero(band,i,quad),i=1,m)
c            read(10,50) (w_aero(band,i,quad),i=1,m)
            read(10,50)
            read(10,50) (g_aero(band,i,quad),i=1,m)
c            read(10,50) (k_cloud(band,i,quad),i=1,m)
            read(10,50)
            read(10,50) (ka_cloud(band,i,quad),i=1,m)
            read(10,50) (kd_cloud(band,i,quad),i=1,m)
c            read(10,50) (w_cloud(band,i,quad),i=1,m)
            read(10,50)
            read(10,50) (g_cloud(band,i,quad),i=1,m)
         enddo ! fin boucle sur quad
      enddo ! fin boucle sur band
      close(10)

c     Debug
c         write(*,*) 'nb_min=',nb_min,' nb_max=',nb_max
c         do band=nb_min,nb_max
c            write(*,*) lambda_lo(band),lambda_hi(band)
c         enddo
c         write(*,*)
c     Debug
      
      if ((l_min_t.eq.1).and.(l_max_t.eq.1)) then
         nbr_nb=nb_max-nb_min+1
         call screen(21,0,0,lambda_lo(nb_min),0.0D+0)
         call screen(22,0,0,lambda_hi(nb_max),0.0D+0)
      endif

      call screen(23,nb_max-nb_min+1,0,0.0D+0,0.0D+0)

      if (rmh.eq.1) then
         call remesh(Nb,Nq)
      endif
      
c     Test numrique sur les valeurs de ka et kd
c     Le but est de ne pas avoir de valeurs trop petites pour viter
c     les bugs numriques (lorsqu'on divise par un ka ou un kd par exemple)

      call look_for_zeroes(Nb,Nq)

c     Calcul du Nombre d'Onde associ  chaque longueur d'onde du fichier
      do band=1,Nb
         no_lo(band)=1.0D+4/lambda_hi(band)
         no_hi(band)=1.0D+4/lambda_lo(band)
      enddo


      open(10,file='./results/mesh_freq.txt')
      write(10,*) ' i - lambda - N.O.'
      do band=1,Nb
         write(10,*) band,lambda_lo(band),no_lo(band)
      enddo
      close(10)
      

 1234 continue
      return
      end
