c     Copyright (C)
c     2000 -2013 Universite Paul Sabatier,
c     2000 -2013 Centre National de la Recherche Scientifique
c     2000 -2013 Ecole des Mines d'Albi
c     2014-2018 Msostar
c
c     This program is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 2, or (at your option)
c     any later version.

c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with this program; if not, write to the Free Software
c     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
c     USA.
c
      subroutine update_temp(arret,sens,Nb,Nq,band,quad,i,j,k0,k1
     &     ,B_em,B_abs,alpha,ksi_abs,w_abs,ksi_out,w_out)

      implicit none

c     Routine de MAJ des flux et PNE pour un rayon
c     
c     Entres :
c       + sens : sens=0 : tir vers le bas ; sens=1 : tir vers le haut
c       + band : numro de la bande troite
c       + quad : numro de la quadrature
c       + i : numro de la maille d'mission du rayon [0,m+1]
c       + j : numro de la maille o se droule l'absorption [0,m+1]
c       + k0 : indice de la fin des tableaux temporaires sens=0
c       + k1 : indice de la fin des tableaux temporaires sens=1
c       + B_em : luminance  l'altitude d'mission du rayon
c       + B_abs : luminance  l'altitude d'absorption du rayon
c       + alpha : direction de parcours du photon en sortie de maille gazeuse
c                 ou aprs rflexion
c       + ksi_abs : nergie absorbe par la maille
c       + w_abs : poids du rayon au moment de l'absorption
c       + ksi_out : nergie sortant de la maille aprs absorption
c       + w_out : poids du rayon aprs l'absorption
c
c     Sorties :
c       + k0 et k1 (mis  jour) : indices de fin des tableaux temporaires
c

      include 'max.inc'
      include 'param.inc'
      include 'data.inc'
      include 'options.inc'
      include 'tab.inc'

      integer i,j,k0,k1,raf1,raf2,p,arret,sens,sec,ind
      double precision B_em,B_abs,alpha,domega
      double precision ksi_abs,w_abs,ksi_out,w_out
      integer Nb,Nq,band,quad,kp,res
      double precision delta_lambda,dlambda
c     Debug
      integer out
c     Debug
      delta_lambda=lambda_hi(band)-lambda_lo(band)

c     MAJ des pne : pne(i,j) = puissance nette change entre les mailles i et j
c     
      call find_sec(alpha,sec,domega)

      if (sens.eq.1) then
         k1=k1+1
         temp_pneh(k1,1)=delta_lambda*(B_abs-B_em)*ksi_abs*w_abs*
     &        w_moy(band,quad)
c     Debug
c         write(*,*) 'delta_lambda=',delta_lambda
c         write(*,*) 'B_abs-B_em=',B_abs-B_em
c         write(*,*) 'ksi_abs=',ksi_abs
c         write(*,*) 'w_abs=',w_abs
c         write(*,*) 'w_moy(',band,',',quad,')=',w_moy(band,quad)
c         write(*,*) 'temp_pneh(',k1,',1)=',temp_pneh(k1,1),j
c     Debug
c     Debug
         call test_nan(temp_pneh(k1,1),out)
         if (out.eq.1) then
            write(*,*) 'from update_temp'
            write(*,*) 'temp_pneh(',k1,',1)=',temp_pneh(k1,1)
            write(*,*) 'delta_lambda=',delta_lambda
            write(*,*) 'B_abs=',B_abs
            write(*,*) 'B_em=',B_em
            write(*,*) 'ksi_abs=',ksi_abs
            write(*,*) 'w_abs=',w_abs
            write(*,*) 'W_moy(',band,',',quad,')=',w_moy(band,quad)
            stop
         endif
c     Debug
         temp_pneh(k1,2)=j
         call numk(Nq,band,quad,kp)
         temp_pneh(k1,3)=dble(kp)
c     MAJ des tableaux temporaires pour les flux
         if (alpha.gt.pi/2.0D+0) then ! alors le rayon monte
            temp_fasch(k1,1)=delta_lambda*B_em*ksi_out*w_out*
     &           w_moy(band,quad)
            temp_fasch(k1,2)=j
            temp_fasch(k1,3)=dble(kp)
            temp_fdesh(k1,1)=0.0D+0
            temp_fdesh(k1,2)=j-1
            temp_fdesh(k1,3)=dble(kp)
            temp_fah_ang(k1,1)=B_em*ksi_out*w_out
c     &           *dtheta/pi
     &           *w_moy(band,quad) ! energie
     &           *delta_lambda
     &           /dabs(dcos(alpha))
     &           /domega
            temp_fah_ang(k1,2)=j ! numro de l'interface
            temp_fah_ang(k1,3)=sec ! numro du secteur angulaire
c     Debug
c            write(*,*) (temp_fah_ang(k1,ind),ind=1,3),' j =',j
c     Debug
            temp_fdh_ang(k1,1)=0.0D+0
            temp_fdh_ang(k1,2)=j-1
            temp_fdh_ang(k1,3)=sec
         else                   ! sinon le rayon descend
            temp_fasch(k1,1)=0.0D+0
            temp_fasch(k1,2)=j
            temp_fasch(k1,3)=dble(kp)
            temp_fdesh(k1,1)=delta_lambda*B_em*ksi_out*w_out*
     &           w_moy(band,quad)
            temp_fdesh(k1,2)=j-1
            temp_fdesh(k1,3)=dble(kp)
            temp_fah_ang(k1,1)=0.0D+0
            temp_fah_ang(k1,2)=j
            temp_fah_ang(k1,3)=sec
            temp_fdh_ang(k1,1)=B_em*ksi_out*w_out
c     &           *dtheta/pi
     &           *w_moy(band,quad) ! energie
     &           *delta_lambda
     &           /dabs(dcos(alpha))
     &           /domega
            temp_fdh_ang(k1,2)=j-1 ! numro de l'interface
            temp_fdh_ang(k1,3)=sec ! numro du secteur angulaire
         endif
      else
         k0=k0+1
         temp_pneb(k0,1)=delta_lambda*(B_abs-B_em)*ksi_abs*w_abs*
     &        w_moy(band,quad)
c     Debug
c         if ((j.eq.3).and.
c     &        ((temp_pneb(k0,1).lt.-9.9951022D-8)
c     &        .or.(temp_pneb(k0,1).gt.-9.9951021D-8))) then
c            write(*,*) 'temp_pneb(',k0,',1)=',temp_pneb(k0,1),j
c            write(*,*) 'delta_lambda=',delta_lambda
c            write(*,*) 'B_abs-B_em=',B_abs-B_em
c            write(*,*) 'ksi_abs=',ksi_abs
c            write(*,*) 'w_abs=',w_abs
c            write(*,*) 'w_moy(',band,',',quad,')=',w_moy(band,quad)
c         endif
c     Debug
         temp_pneb(k0,2)=j
         call numk(Nq,band,quad,kp)
         temp_pneb(k0,3)=dble(kp)
c     MAJ des tableaux temporaires pour les flux
         if (alpha.gt.pi/2.0D+0) then ! alors le rayon monte
            temp_fascb(k0,1)=delta_lambda*B_em*ksi_out*w_out*
     &           w_moy(band,quad)
            temp_fascb(k0,2)=j
            temp_fascb(k0,3)=dble(kp)
            temp_fdesb(k0,1)=0.0D+0
            temp_fdesb(k0,2)=j-1
            temp_fdesb(k0,3)=dble(kp)
            temp_fab_ang(k0,1)=B_em*ksi_out*w_out
c     &           *dtheta/pi
     &           *w_moy(band,quad) ! energie
     &           *delta_lambda
     &           /dabs(dcos(alpha))
     &           /domega
            temp_fab_ang(k0,2)=j ! numro de l'interface
            temp_fab_ang(k0,3)=sec ! numro du secteur angulaire
            temp_fdb_ang(k0,1)=0.0D+0
            temp_fdb_ang(k0,2)=j-1
            temp_fdb_ang(k0,3)=sec
         else                   ! sinon le rayon descend
            temp_fascb(k0,1)=0.0D+0
            temp_fascb(k0,2)=j
            temp_fascb(k0,3)=dble(kp)
            temp_fdesb(k0,1)=delta_lambda*B_em*ksi_out*w_out*
     &           w_moy(band,quad)
            temp_fdesb(k0,2)=j-1
            temp_fdesb(k0,3)=dble(kp)
            temp_fab_ang(k0,1)=0.0D+0
            temp_fab_ang(k0,2)=j
            temp_fab_ang(k0,3)=sec
            temp_fdb_ang(k0,1)=B_em*ksi_out*w_out
c     &           *dtheta/pi
     &           *w_moy(band,quad) ! energie
     &           *delta_lambda
     &           /dabs(dcos(alpha))
     &           /domega
            temp_fdb_ang(k0,2)=j-1 ! numro de l'interface
            temp_fdb_ang(k0,3)=sec ! numro du secteur angulaire
         endif
      endif

      if ((k0.gt.Pmax).or.(k1.gt.Pmax)) then
         call screen(1106,0,0,0.0D+0,0.0D+0)
c     Debug
         write(*,*) 'ksi_out:',ksi_out
         write(*,*) 'sens:',sens
         if (sens.eq.1) then
            do i=1,70
               write(*,*) temp_pneh(i,1),temp_pneh(i,2),temp_pneh(i,3)
            enddo
         else
            do i=1,70
               write(*,*) temp_pneh(i,1),temp_pneh(i,2),temp_pneh(i,3)
            enddo
         endif
c     Debug
         stop
      endif
      
10000 return
      end

      

      subroutine update_results(md,k0,k1)

      implicit none

c     Routine de MAJ des rsultats (une fois qu'un rayon a termin son parcours)
c     
c     Entres
c       + md : numro de la maille de dpart du rayon
c       + k0 : indice de fin des tableaux temporaires sens=0
c       + k1 : indice de fin des tableaux temporaires sens=1
c

      include 'max.inc'
      include 'param.inc'
      include 'data.inc'
      include 'options.inc'
      include 'tab.inc'

      integer i,j,k,k0,k1,p,md,res,s,sec
      double precision alpha1,beta1(0:Nmax+2)
      double precision alpha0,beta0(0:Nmax+2)
c     Debug
      integer out
c     Debug
     
      alpha0=0.0D+0
      do p=0,m+1
         beta0(p)=0.0D+0
      enddo
      alpha1=0.0D+0
      do p=0,m+1
         beta1(p)=0.0D+0
      enddo

c     Debug
c      do p=1,k1
c         write(*,*) (temp_fah_ang(p,j),j=1,3)
c      enddo
c     Debug
c     Debug
c      write(*,*) 'k0=',k0,' k1=',k1
c     Debug

      if (k1.gt.0) then
         do p=1,k1
            j=int(temp_pneh(p,2))
c     Seulement si on a choisi de calculer les PNE monochromatiques,
c     pour ne pas utiliser de RAM supplmentaire
            if (cpm.eq.1) then
               k=int(temp_pneh(p,3))
c               sum_pnefh(md,j,k)=sum_pnefh(md,j,k)+
c     &              sngl(temp_pneh(p,1))
c               sum2_pnefh(md,j,k)=sum2_pnefh(md,j,k)+
c     &              sngl(temp_pneh(p,1))**2
c     Debug
               call test_nan(temp_pneh(p,1),out)
               if (out.eq.1) then
                  write(*,*) 'from update_results'
                  write(*,*) 'temp_pneh(',p,',1)=',temp_pneh(p,1)
                  stop
               endif
c     Debug
               sum_pnefh(md,j,k)=sum_pnefh(md,j,k)+
     &              temp_pneh(p,1)
               sum2_pnefh(md,j,k)=sum2_pnefh(md,j,k)+
     &              temp_pneh(p,1)**2
            endif
c
            sum_pneh(md,j)=sum_pneh(md,j)+temp_pneh(p,1)
            sum2_pneh(md,j)=sum2_pneh(md,j)+temp_pneh(p,1)**2
c     Debug
c            write(*,*) 'temp_pneh(',p,',1)=',temp_pneh(p,1),
c     &           ' alpha1=',alpha1
c     Debug
            alpha1=alpha1+temp_pneh(p,1)
            beta1(j)=beta1(j)-temp_pneh(p,1)
            if ((md.eq.0).and.(j.eq.m+1)) then
               sum_bil_se_h(k)=sum_bil_se_h(k)-temp_pneh(p,1)
               sum2_bil_se_h(k)=sum2_bil_se_h(k)+temp_pneh(p,1)**2
            else if ((md.eq.0).and.(j.ne.0)) then
               sum_bil_gs_h(j,k)=sum_bil_gs_h(j,k)-temp_pneh(p,1)
               sum2_bil_gs_h(j,k)=sum2_bil_gs_h(j,k)+temp_pneh(p,1)**2
            else if ((md.eq.m+1).and.(j.ne.m+1)) then
               sum_bil_ge_h(j,k)=sum_bil_ge_h(j,k)-temp_pneh(p,1)
               sum2_bil_ge_h(j,k)=sum2_bil_ge_h(j,k)+temp_pneh(p,1)**2
            else
               sum_bil_gg_h(j,k)=sum_bil_gg_h(j,k)-temp_pneh(p,1)
               sum2_bil_gg_h(j,k)=sum2_bil_gg_h(j,k)+temp_pneh(p,1)**2
            endif

c     Pour les flux
            if (cpm.eq.1) then
               j=int(temp_fasch(p,2))
               k=int(temp_fasch(p,3))
               sum_fafh(j,k)=sum_fafh(j,k)+temp_fasch(p,1)
               sum2_fafh(j,k)=sum2_fafh(j,k)+temp_fasch(p,1)**2
               j=int(temp_fdesh(p,2))
               k=int(temp_fdesh(p,3))
               sum_fdfh(j,k)=sum_fdfh(j,k)+temp_fdesh(p,1)
               sum2_fdfh(j,k)=sum2_fdfh(j,k)+temp_fdesh(p,1)**2
            endif
            j=int(temp_fasch(p,2))
            sum_fasch(md,j)=sum_fasch(md,j)+temp_fasch(p,1)
            sum2_fasch(md,j)=sum2_fasch(md,j)+temp_fasch(p,1)**2
            j=int(temp_fdesh(p,2))
            sum_fdesh(md,j)=sum_fdesh(md,j)+temp_fdesh(p,1)
            sum2_fdesh(md,j)=sum2_fdesh(md,j)+temp_fdesh(p,1)**2
c
            j=int(temp_fah_ang(p,2))
            sec=int(temp_fah_ang(p,3))
            sum_fah_ang(j,sec)=sum_fah_ang(j,sec)+temp_fah_ang(p,1)
c     Debug
c            write(*,*) 'sum_fah_ang(',j,',',sec,')=',sum_fah_ang(j,sec)
c     Debug
            sum2_fah_ang(j,sec)=sum2_fah_ang(j,sec)+temp_fah_ang(p,1)**2
            j=int(temp_fdh_ang(p,2))
            sec=int(temp_fdh_ang(p,3))
            sum_fdh_ang(j,sec)=sum_fdh_ang(j,sec)+temp_fdh_ang(p,1)
            sum2_fdh_ang(j,sec)=sum2_fdh_ang(j,sec)+temp_fdh_ang(p,1)**2
         enddo
         sum_bilh(md,k)=sum_bilh(md,k)+alpha1
         sum2_bilh(md,k)=sum2_bilh(md,k)+alpha1**2
         do p=0,m+1
            sum_bil_rev_h(p,k)=sum_bil_rev_h(p,k)+beta1(p)
c     Debug
c            write(*,*) 'sum_bil_rev_h(',p,',',k,')=',sum_bil_rev_h(p,k)
c     Debug
            sum2_bil_rev_h(p,k)=sum2_bil_rev_h(p,k)+beta1(p)**2
         enddo
      endif
      if (k0.gt.0) then
         do p=1,k0
            j=int(temp_pneb(p,2))
            k=int(temp_pneb(p,3))
c     Debug
c            write(*,*) 'j=',j,' k=',k
c     Debug
c     Seulement si on a choisi de calculer les PNE monochromatiques,
c     pour ne pas utiliser de RAM supplmentaire
            if (cpm.eq.1) then
c               sum_pnefb(md,j,k)=sum_pnefb(md,j,k)+
c     &              sngl(temp_pneb(p,1))
c               sum2_pnefb(md,j,k)=sum2_pnefb(md,j,k)+
c     &              sngl(temp_pneb(p,1))**2
               sum_pnefb(md,j,k)=sum_pnefb(md,j,k)+
     &              temp_pneb(p,1)
               sum2_pnefb(md,j,k)=sum2_pnefb(md,j,k)+
     &              temp_pneb(p,1)**2
            endif
            sum_pneb(md,j)=sum_pneb(md,j)+temp_pneb(p,1)
            sum2_pneb(md,j)=sum2_pneb(md,j)+temp_pneb(p,1)**2
            alpha0=alpha0+temp_pneb(p,1)
c     Debug
c            write(*,*) 'temp_pneb(',p,',1)=',temp_pneb(p,1),
c     &           ' alpha0=',alpha0
c     Debug
            beta0(j)=beta0(j)-temp_pneb(p,1)
            if ((md.eq.0).and.(j.eq.m+1)) then
               sum_bil_se_b(k)=sum_bil_se_b(k)-temp_pneb(p,1)
               sum2_bil_se_b(k)=sum2_bil_se_b(k)+temp_pneb(p,1)**2
            else if ((md.eq.0).and.(j.ne.0)) then
               sum_bil_gs_b(j,k)=sum_bil_gs_b(j,k)-temp_pneb(p,1)
               sum2_bil_gs_b(j,k)=sum2_bil_gs_b(j,k)+temp_pneb(p,1)**2
            else if ((md.eq.m+1).and.(j.ne.m+1)) then
               sum_bil_ge_b(j,k)=sum_bil_ge_b(j,k)-temp_pneb(p,1)
               sum2_bil_ge_b(j,k)=sum2_bil_ge_b(j,k)+temp_pneb(p,1)**2
            else
               sum_bil_gg_b(j,k)=sum_bil_gg_b(j,k)-temp_pneb(p,1)
               sum2_bil_gg_b(j,k)=sum2_bil_gg_b(j,k)+temp_pneb(p,1)**2
            endif

c     Pour les flux
            if (cpm.eq.1) then
               j=int(temp_fascb(p,2))
               k=int(temp_fascb(p,3))
               sum_fafb(j,k)=sum_fafb(j,k)+temp_fascb(p,1)
               sum2_fafb(j,k)=sum2_fafb(j,k)+temp_fascb(p,1)**2
               j=int(temp_fdesb(p,2))
               k=int(temp_fdesb(p,3))
               sum_fdfb(j,k)=sum_fdfb(j,k)+temp_fdesb(p,1)
               sum2_fdfb(j,k)=sum2_fdfb(j,k)+temp_fdesb(p,1)**2
            endif
            j=int(temp_fascb(p,2))
            sum_fascb(md,j)=sum_fascb(md,j)+temp_fascb(p,1)
            sum2_fascb(md,j)=sum2_fascb(md,j)+temp_fascb(p,1)**2
            j=int(temp_fdesb(p,2))
            sum_fdesb(md,j)=sum_fdesb(md,j)+temp_fdesb(p,1)
            sum2_fdesb(md,j)=sum2_fdesb(md,j)+temp_fdesb(p,1)**2
c
            j=int(temp_fab_ang(p,2))
            sec=int(temp_fab_ang(p,3))
            sum_fab_ang(j,sec)=sum_fab_ang(j,sec)+temp_fab_ang(p,1)
            sum2_fab_ang(j,sec)=sum2_fab_ang(j,sec)+temp_fab_ang(p,1)**2
            j=int(temp_fdb_ang(p,2))
            sec=int(temp_fdb_ang(p,3))
            sum_fdb_ang(j,sec)=sum_fdb_ang(j,sec)+temp_fdb_ang(p,1)
            sum2_fdb_ang(j,sec)=sum2_fdb_ang(j,sec)+temp_fdb_ang(p,1)**2
         enddo ! sur p
c     Debug
c         write(*,*) 'alpha0=',alpha0
c     Debug
         sum_bilb(md,k)=sum_bilb(md,k)+alpha0
c     Debug
c         write(*,*) 'sum_bilb(',md,',',k,')=',sum_bilb(md,k)
c     Debug
         sum2_bilb(md,k)=sum2_bilb(md,k)+alpha0**2
         do p=0,m+1
            sum_bil_rev_b(p,k)=sum_bil_rev_b(p,k)+beta0(p)
c     Debug
c            write(*,*) 'sum_bil_rev_b(',p,',',k,')=',sum_bil_rev_b(p,k)
c     Debug
            sum2_bil_rev_b(p,k)=sum2_bil_rev_b(p,k)+beta0(p)**2
         enddo
      endif

c     01/03/2007
c     BUG MAJEUR dtect: lorsqu'on reprend un calcul interrompu, la
c     variable "paq_min" prend la valeur du numro du dernier vnement
c     alatoire effectu (+1)... et elle n'est jamais rinitialise ! Donc toutes les
c     mailles suivantes ne tireront que 'n_max-paq_min' vnements ! C'est
c     une vritable honte de ne pas y avoir pens. Je corrige a illico:
      paq_min=1

      return
      end
