c     Copyright (C)
c     2000 -2013 Universite Paul Sabatier,
c     2000 -2013 Centre National de la Recherche Scientifique
c     2000 -2013 Ecole des Mines d'Albi
c     2014-2018 Msostar
c
c     This program is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 2, or (at your option)
c     any later version.

c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with this program; if not, write to the Free Software
c     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
c     USA.
c
c.......................................................................
c
c     generateur de nombres aleatoires uniformes entre 0 et 1
c     theoriquement compatible avec toutes les ecrans de la creation 
c
c....................................................................... 
      real function urand(iy)
      integer iy
c.......................................................................
c     urand is a uniform random number generator based on theory and 
c     suggestions given in d.e. knuth (1969), vol 2.
c     the integer iy should be initialized to an arbitrary integer prior
c     to the first call to urand. the calling program should not alter
c     the value of iy between subsequent calls to urand. values of urand
c     will be returned in the interval (0,1).
c.......................................................................
      integer ia,ic,itwo,m2,m,mic
      double precision halfm
      real s
      double precision datan,dsqrt
      m2=0
      itwo=2

      if (m2.ne.0) go to 20
c
c     if first entry, compute machine integer word length
c
      m=1
 10   m2=m
      m=itwo*m2
      if (m.gt.m2) goto 10
      halfm=m2
c
c     compute multiplier and increment for linear congruential method
c
      ia=8*idint(halfm*datan(1.d0)/8.d0)+5
      ic=2*idint(halfm*(0.5d0-dsqrt(3.d0)/6.d0))+1
      mic=(m2-ic)+m2
c
c     s is the scale factor for converting to floating point
c
      s=0.5/halfm
c
c     compute next random number
c
 20   iy=iy*ia
c
c     the following statement is for computers which do not allow
c     integer overflow on addition 
c
      if (iy.gt.mic) iy=(iy-m2)-m2
c
      iy=iy+ic
c
c     the following statement is for computers where the word length
c     for addition is greater than for multiplication
c
      if (iy/2.gt.m2) iy=(iy-m2)-m2
c
c     the following statement is for computers where integer overflow
c     affects the sign bit
c
      if (iy.lt.0) iy=(iy+m2)+m2
      urand=float(iy)*s
c.......................................................................
      return
      end
